/* linux/arch/arm/mach-s3c2410/mach-w5300e01.c
 *
 * Copyright (C) 2008 WIZnet
 * All rights reserved.
 *
 * @Author: Taehun Kim
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * @History:
 * derived from linux/arch/arm/mach-s3c2410/mach-bast.c, written by
 * Ben Dooks <ben@simtec.co.uk>
 *
 ***********************************************************************/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/serial_core.h>
#include <linux/platform_device.h>

#include <asm/arch/regs-gpio.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#include <asm/plat-s3c/nand.h>
#include <asm/plat-s3c/regs-serial.h>
#include <asm/plat-s3c24xx/devs.h>
#include <asm/plat-s3c24xx/cpu.h>

#include <asm/plat-s3c24xx/common-smdk.h>
#include <asm/plat-s3c24xx/pm.h>

static struct map_desc w5300e01_iodesc[] __initdata = {
  { 0xf0000000, __phys_to_pfn(S3C2410_CS2), SZ_1M, MT_DEVICE },
	{ 0xf8000000, __phys_to_pfn(S3C2410_CS3), SZ_1M, MT_DEVICE }
};

#define UCON S3C2410_UCON_DEFAULT
#define ULCON S3C2410_LCON_CS8 | S3C2410_LCON_PNONE | S3C2410_LCON_STOPB
#define UFCON S3C2410_UFCON_RXTRIG8 | S3C2410_UFCON_FIFOMODE

static struct s3c2410_uartcfg w5300e01_uartcfgs[] __initdata = {
	[0] = {
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[1] = {
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[2] = {
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	}
};

static struct mtd_partition w5300e01_mtd_partitions[] = {
	[0] = {
		.name	= "Bootloader",
		.size	= 0x20000,
		.offset	= 0,
		.mask_flags = MTD_WRITEABLE,
	}, 
  [1] = {
		.name	= "Boot Param",
		.size	= 0x20000,
		.offset	= MTDPART_OFS_APPEND,
	}, 
  [2] = {
		.name	= "Kernel",
		.size	= 0x3C0000,
		.offset	= MTDPART_OFS_APPEND,
	}, 
  [3] = {
		.name	= "Ramdisk",
		.size	= 0x1000000,
		.offset	= MTDPART_OFS_APPEND,
	}, 
  [4] = {
		.name	= "JFFS2 FileSystem",
		.size	= MTDPART_SIZ_FULL,
		.offset	= MTDPART_OFS_APPEND,
	}, 
};

static struct s3c2410_nand_set w5300e01_nand_sets[] = {
	[0] = {
		.name		    = "W5300E01",
		.nr_chips	    = 1,
		.nr_partitions	= ARRAY_SIZE(w5300e01_mtd_partitions),
		.partitions	    = w5300e01_mtd_partitions,
	},
};

static struct s3c2410_platform_nand w5300e01_nand_info = {
	.tacls		    = 20,
	.twrph0		    = 60,
	.twrph1		    = 30,
	.nr_sets	    = ARRAY_SIZE(w5300e01_nand_sets),
	.sets		    = w5300e01_nand_sets,
};

static struct platform_device *w5300e01_devices[] __initdata = {
  &s3c_device_nand,
	&s3c_device_usb,
	&s3c_device_wdt,
	&s3c_device_i2c,
	&s3c_device_iis,
};

static void __init w5300e01_map_io(void)
{
	s3c24xx_init_io(w5300e01_iodesc, ARRAY_SIZE(w5300e01_iodesc));
	s3c24xx_init_clocks(0);
	s3c24xx_init_uarts(w5300e01_uartcfgs, ARRAY_SIZE(w5300e01_uartcfgs));
}

static void __init w5300e01_init(void)
{
	s3c_device_nand.dev.platform_data = &w5300e01_nand_info;
	platform_add_devices(w5300e01_devices, ARRAY_SIZE(w5300e01_devices));

  /* W5300 interrupt pin */
  s3c2410_gpio_cfgpin(S3C2410_GPF0, S3C2410_GPF0_EINT0);

  s3c2410_gpio_cfgpin(S3C2410_GPF4, S3C2410_GPF4_OUTP);
	s3c2410_gpio_cfgpin(S3C2410_GPF5, S3C2410_GPF5_OUTP);
	s3c2410_gpio_cfgpin(S3C2410_GPF6, S3C2410_GPF6_OUTP);
	s3c2410_gpio_cfgpin(S3C2410_GPF7, S3C2410_GPF7_OUTP);

	s3c2410_gpio_setpin(S3C2410_GPF0, 1);
	s3c2410_gpio_setpin(S3C2410_GPF4, 1);
	s3c2410_gpio_setpin(S3C2410_GPF5, 1);
	s3c2410_gpio_setpin(S3C2410_GPF6, 1);
	s3c2410_gpio_setpin(S3C2410_GPF7, 1);

	s3c2410_pm_init();
}

MACHINE_START(W5300E01, "WIZnet W5300E01 Board")
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,
	.map_io		= w5300e01_map_io,
	.init_irq	= s3c24xx_init_irq,
	.init_machine	= w5300e01_init,
	.timer		= &s3c24xx_timer,
MACHINE_END


